<?php

namespace Drupal\masquerade_as_role\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Password\PasswordGeneratorInterface;
use Drupal\masquerade\Form\MasqueradeForm;
use Drupal\masquerade\Masquerade;
use Drupal\user\Entity\Role;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form builder for the masquerade as role form.
 */
final class MasqueradeAsRoleForm extends MasqueradeForm {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The masquerade service.
   *
   * @var \Drupal\masquerade\Masquerade
   */
  protected $masquerade;

  /**
   * Interface to generate passwords.
   *
   * @var \Drupal\Core\Password\PasswordGeneratorInterface
   */
  protected $passwordGenerator;

  /**
   * Constructs a MasqueradeForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $etm
   *   The entity type manager.
   * @param \Drupal\masquerade\Masquerade $masquerade
   *   The masquerade service.
   * @param \Drupal\Core\Password\PasswordGeneratorInterface $passwordGenerator
   *   The password generator service.
   */
  public function __construct(EntityTypeManagerInterface $etm, Masquerade $masquerade, PasswordGeneratorInterface $passwordGenerator) {
    $this->entityTypeManager = $etm;
    $this->masquerade = $masquerade;
    $this->passwordGenerator = $passwordGenerator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container):self {
    return new self(
      $container->get('entity_type.manager'),
      $container->get('masquerade'),
      $container->get('password_generator')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @param mixed[] $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return mixed[]
   *   The form array.
   */
  public function buildForm(array $form, FormStateInterface $form_state):array {
    // @todo Change the autogenerated stub.
    $form = parent::buildForm($form, $form_state);
    $form['autocomplete']['masquerade_as']['#required'] = FALSE;
    $roles = Role::loadMultiple();
    $roleOptions = [];
    foreach ($roles as $key => $role) {
      // No need to masquerade using anonymous role.
      if ($key !== 'anonymous') {
        $roleOptions[$key] = $role->label();
      }
    }
    $form['autocomplete']['masquerade_as']['#description'] = $this->t('Enter the username to masquerade as.<br>');
    $form['autocomplete']['masquerade_as_role'] = [
      '#type' => 'select',
      '#options' => $roleOptions,
      '#description' => $this->t('Alternatively, select one or several role(s) to masquerade as.<br>'),
      '#multiple' => TRUE,
    ];
    // Set appropriate weights.
    $form['autocomplete']['masquerade_as']['#weight'] = 1;
    $form['autocomplete']['masquerade_as_role']['#weight'] = 2;
    $form['submit']['#weight'] = 3;
    $form['#attached']['library'][] = 'masquerade_as_role/masquerade_as_role';

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * @param mixed[] $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) :void {
    $rids = $form_state->getValue('masquerade_as_role');
    if (empty($rids)) {
      $name = $form_state->getValue('masquerade_as');
      if (empty($name)) {
        $form_state->setErrorByName('masquerade_as', $this->t('The user does not exist. Please enter a valid username.'));
        return;
      }
      else {
        parent::validateForm($form, $form_state);
        return;
      }
    }
    $edit = [];
    $edit['name'] = substr(md5(microtime()), rand(0, 26), 8);
    $edit['mail'] = $edit['name'] . 'masquerade_as_role@example.com';
    $edit['pass'] = $this->passwordGenerator->generate();
    $edit['roles'] = array_keys($rids);
    $edit['status'] = 1;
    $target_account = $this->entityTypeManager
      ->getStorage('user')->create($edit);
    $target_account->save();
    if ($error = masquerade_switch_user_validate($target_account)) {
      $form_state->setErrorByName('masquerade_as', $error);
    }
    else {
      $form_state->setValue('masquerade_target_account', $target_account);
    }
  }

}
